import { FrameworkPluginFactory } from '@previewjs/core';
import playwright from 'playwright';

declare function generateScreenshots({ page, frameworkPlugins, filePathPattern, cwd, generateScreenshotPath, onError, onScreenshotGenerated, }: {
    /**
     * A Playwright page which will be used to load the preview.
     *
     * This can be used to emulate different viewports, for example a Pixel 2 mobile phone:
     * ```js
     * const browser = await playwright.chromium.launch();
     * const context = await browser.newContext(playwright.devices["Pixel 2"]);
     * const page = await context.newPage();
     * ```
     */
    page: playwright.Page;
    /**
     * A list of Preview.js framework plugins (typically just one).
     *
     * Example:
     * ```js
     * [(await import("@previewjs/plugin-react")).default]
     * ```
     */
    frameworkPlugins: FrameworkPluginFactory[];
    /**
     * A file path pattern used to select files from which to render components and stories.
     *
     * Follows [minimatch](https://github.com/isaacs/minimatch) syntax.
     *
     * Example: `"**\*.stories.{js,ts,js,tsx}"`
     */
    filePathPattern: string;
    /**
     * Workspace root directory (should contain package.json).
     */
    cwd?: string;
    /**
     * Callback invoked to generate the path at which the screenshot will be generated.
     *
     * Example:
     * ```js
     * generateScreenshotPath({ filePath, name }) {
     *   return `${filePath}-${name}.png`;
     * }
     * ```
     */
    generateScreenshotPath: (options: {
        filePath: string;
        name: string;
    }) => string;
    /**
     * Callback invoked when a component or story failed to render.
     *
     * Return `true` to halt or `false` to ignore the failure and keep generating other screenshots.
     */
    onError?: (error: Error, options: {
        filePath: string;
        name: string;
    }) => boolean;
    /**
     * Callback invoked after a screenshot has been successfully generated.
     */
    onScreenshotGenerated?: (options: {
        filePath: string;
        name: string;
        screenshotPath: string;
    }) => void;
}): Promise<void>;

export { generateScreenshots };
